/// <reference path="../../../../SDL/TypeScript/SDL.Client.UI.Core.Knockout/ViewModels/ViewModelBase.d.ts" />
/// <reference path="../../../../SDL.Web.UI.PublicationMappings.Model/Client/ItemTypes/Publication.d.ts" />
/// <reference path="../../../../SDL.Web.UI.PublicationMappings.Model/Client/Lists/ListCdTopologies.d.ts" />
/// <reference path="../../../../SDL.Web.UI.PublicationMappings.Model/Client/Lists/ListWebApplications.d.ts" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var SDL;
(function (SDL) {
    var Web;
    (function (Web) {
        var UI;
        (function (UI) {
            var PublicationMappings;
            (function (PublicationMappings) {
                var ViewModels;
                (function (ViewModels) {
                    eval(SDL.Client.Types.OO.enableCustomInheritance);
                    var Mappings = (function (_super) {
                        __extends(Mappings, _super);
                        function Mappings() {
                            _super.call(this);
                            this._existingMappings = [];
                            this._webApplicationIds = [];
                            this._activeTopology = "";
                            this._publicationId = "";
                            this._topologyTypeId = "";
                            this._mappingsLoadFinished = false;
                            this._bptsLoadFinished = false;
                            this._topologyManagerLoadFailed = false;
                            this.activeTopologyPurposes = ko.observableArray([]);
                            this.mappings = ko.observableArray([]);
                            this.availableTopologies = ko.observableArray([]);
                            this.businessProcessTypes = ko.observable([]);
                            this.webApplications = ko.observableArray([]);
                            this.mappingTooltipTargetElement = ko.observable(null);
                            this.mappingTooltipVisible = ko.observable(false);
                            this.selectedBusinessProcessType = ko.observable(null);
                            this.selectedTopology = ko.observable(null);
                            this.selectedEnvironment = ko.observable(null);
                            this.isConfigurationEditable = ko.observable(false);
                            this.isBusinessProcessTypeDropdownDisabled = ko.observable(true);
                            this.isSelectTopologyDialogVisible = ko.observable(false);
                            this.isRemoveMappingDialogVisible = ko.observable(false);
                            this.isTopologyManagerAvailable = ko.observable(false);
                            this.isLoading = ko.observable(false);
                            this.webApplicationsLoading = ko.observable(false);
                            this.topologiesLoadFailed = ko.observable(false);
                            var publication = $display.getView().getItem();
                            this._publicationId = publication.getId();
                            this._setupComputedObservables();
                            this._setupSubscriptions();
                            this.isLoading(true);
                            if (publication.isLoaded()) {
                                this._initialize();
                            }
                            else {
                                $evt.addEventHandler(publication, "load", this.getDelegate(this._initialize));
                            }
                            this._loadBusinessProcessTypes();
                        }
                        Mappings.prototype._setupSubscriptions = function () {
                            var _this = this;
                            this.selectedEnvironment.subscribe(function (newPurpose) {
                                if (!newPurpose) {
                                    return;
                                }
                                _this._setActiveMappingsDirty();
                                var mapping = _this._getMapping(newPurpose);
                                if (mapping != null && !mapping.isEmpty()) {
                                    _this._loadWebApplications(mapping.environmentId);
                                }
                            });
                            this.selectedBusinessProcessType.subscribe(function (newBpt) {
                                _this._activeTopology = "";
                                var publication = _this._getPublication();
                                publication.setBusinessProcessType(newBpt, _this);
                                _this._updateData();
                            });
                        };
                        Mappings.prototype._setupComputedObservables = function () {
                            var _this = this;
                            this.isEditorVisible = ko.pureComputed(function () {
                                return _this.isTopologyManagerAvailable() && _this.selectedBusinessProcessType() && _this.businessProcessTypes().length > 0;
                            });
                        };
                        Mappings.prototype._getPublication = function () {
                            return $models.getItem(this._publicationId);
                        };
                        Mappings.prototype._initialize = function () {
                            var publication = this._getPublication();
                            $evt.removeEventHandler(publication, "load", this.getDelegate(this._initialize));
                            $evt.addEventHandler(publication, "change", this.getDelegate(this._onPublicationChange));
                            $evt.addEventHandler(publication, "collectdata", this.getDelegate(this._onPublicationCollectData));
                            $evt.addEventHandler(publication, "validate", this.getDelegate(this._onPublicationValidate));
                            $evt.addEventHandler(publication, "load", this.getDelegate(this._onPublicationLoad));
                            this.isBusinessProcessTypeDropdownDisabled(!publication.hasBptManagementRights());
                            this.isTopologyManagerAvailable(publication.areMappingsAccessible());
                            this._updateSelectedBusinessProcessTypeValue();
                            if (this.isTopologyManagerAvailable()) {
                                this._loadTopologyMappings();
                            }
                            else {
                                $messages.registerError(this.localize("tabs.mappings.topologyManagerUnavailableTitle"), this.localize("tabs.mappings.topologyManagerUnavailableDetails"), null, window, window, false);
                                this.isLoading(false);
                            }
                        };
                        Mappings.prototype._loadBusinessProcessTypes = function () {
                            var publication = this._getPublication();
                            var listBusinessProcessTypes = publication.getListBusinessProcessTypes();
                            this._executeOnceWhenItemLoaded(listBusinessProcessTypes, this.getDelegate(this._onListBusinessProcessTypesLoad), this.getDelegate(this._onListBusinessProcessTypesLoadFailed));
                        };
                        Mappings.prototype._onListBusinessProcessTypesLoad = function (list) {
                            this._bptsLoadFinished = true;
                            this._populateBusinessProcessTypesDropdown();
                        };
                        Mappings.prototype._onListBusinessProcessTypesLoadFailed = function (event) {
                            this.isBusinessProcessTypeDropdownDisabled(true);
                        };
                        Mappings.prototype._populateBusinessProcessTypesDropdown = function () {
                            this._updateListBusinessProcessTypes();
                            this._updateData();
                        };
                        Mappings.prototype._updateListBusinessProcessTypes = function () {
                            var publication = this._getPublication();
                            var listBusinessProcessTypes = publication.getListBusinessProcessTypes();
                            var bptListXmlDocument = listBusinessProcessTypes.getXmlDocument();
                            var itemNodes = $xml.selectNodes(bptListXmlDocument, "/tcm:ListBusinessProcessTypes/tcm:Item");
                            var items = Array.prototype.slice.call(itemNodes);
                            var bptArray = items.map(function (item) { return ({
                                value: $xml.getInnerText(item, "@ID"),
                                text: $xml.getInnerText(item, "@Title")
                            }); });
                            bptArray.unshift({ value: null, text: this.localize("tabs.mappings.none") });
                            this.businessProcessTypes(bptArray);
                        };
                        Mappings.prototype._loadTopologyMappings = function () {
                            var listTopologyMappings = $models.getItem($const.TCMROOT).getListTopologyMappings();
                            listTopologyMappings.unload();
                            this._executeOnceWhenItemLoaded(listTopologyMappings, this.getDelegate(this._onListTopologyMappingsLoad), this.getDelegate(this._onListTopologyMappingsLoadFailed));
                        };
                        Mappings.prototype._onListTopologyMappingsLoad = function (list) {
                            var publication = this._getPublication();
                            this.isConfigurationEditable(publication.areMappingsEditable());
                            this._mappingsLoadFinished = true;
                            this._updateExistingTopologyMappings();
                        };
                        Mappings.prototype._onListTopologyMappingsLoadFailed = function (event) {
                            this.isConfigurationEditable(false);
                            this._topologyManagerLoadFailed = true;
                            this._mappingsLoadFinished = true;
                            this._updateData();
                        };
                        Mappings.prototype._updateExistingTopologyMappings = function () {
                            var listTopologyMappings = $models.getItem($const.TCMROOT).getListTopologyMappings();
                            var xmlDoc = listTopologyMappings.getXmlDocument();
                            var itemsNode = $xml.selectNodes(xmlDoc, "/ttm:*/ttm:TopologyMapping");
                            var items = Array.prototype.slice.call(itemsNode);
                            this._existingMappings = items.map(function (item) {
                                return ({
                                    publicationId: item.getAttribute("PublicationId"),
                                    environmentPurpose: item.getAttribute("EnvironmentPurpose"),
                                    primaryMappedUrl: item.getAttribute("PrimaryMappedUrl")
                                });
                            });
                            this._updateData();
                        };
                        Mappings.prototype._loadTopologies = function () {
                            var _this = this;
                            this.availableTopologies([]);
                            var publication = this._getPublication();
                            var listBusinessProcessTypes = publication.getListBusinessProcessTypes();
                            this._topologyTypeId = listBusinessProcessTypes.getTopologyTypeForBpt(publication.getBusinessProcessTypeId());
                            this._webApplicationIds = this.mappings()
                                .filter(function (mapping) { return !mapping.isDeleted()
                                && !!mapping.webApplicationId()
                                && _this.activeTopologyPurposes.indexOf(mapping.environmentPurpose) !== -1; })
                                .map(function (mapping) { return mapping.webApplicationId(); });
                            this.topologiesLoadFailed(false);
                            this.isLoading(true);
                            var listTopologies = $models.getItem($const.TCMROOT)
                                .getListTopologies(this._topologyTypeId, this._webApplicationIds.length > 0 ? this._webApplicationIds : null);
                            this._executeOnceWhenItemLoaded(listTopologies, this.getDelegate(this._onListTopologiesLoad), this.getDelegate(this._onListTopologiesLoadFailed));
                        };
                        Mappings.prototype._onListTopologiesLoad = function (list) {
                            this._processListTopologies();
                            this.isLoading(false);
                        };
                        Mappings.prototype._onListTopologiesLoadFailed = function (event) {
                            this.topologiesLoadFailed(true);
                            this.isLoading(false);
                        };
                        Mappings.prototype._processListTopologies = function () {
                            this._updateTopologies();
                            var selectedMapping = this._getMapping(this.selectedEnvironment());
                            if (this.availableTopologies().length > 2 || !selectedMapping.isEmpty()) {
                                this._showSelectTopologyDialog();
                                return;
                            }
                            if (this.availableTopologies()[1]) {
                                this.selectedTopology(this.availableTopologies()[1].value);
                                this.onSelectTopologyDialogButtonClick();
                            }
                            else {
                                selectedMapping.isConfigurationInvalid(true);
                            }
                        };
                        Mappings.prototype._updateTopologies = function () {
                            var listTopologies = $models.getItem($const.TCMROOT).getListTopologies(this._topologyTypeId, this._webApplicationIds);
                            var listTopologiesXmlDoc = listTopologies.getXmlDocument();
                            var topologyNodes = $xml.selectNodes(listTopologiesXmlDoc, "/pm:CdTopologies/pm:CdTopology");
                            var items = Array.prototype.slice.call(topologyNodes);
                            var topologiesArray = items.map(function (item) {
                                var envNodes = $xml.selectNodes(item, "pm:CdEnvironment/@Id");
                                var envItems = Array.prototype.slice.call(envNodes);
                                return {
                                    value: $xml.getInnerText(item, "@Id"),
                                    text: $xml.getInnerText(item, "@Name"),
                                    environments: envItems.map(function (env) { return $xml.getInnerText(env); })
                                };
                            });
                            topologiesArray.unshift({ value: "", text: this.localize("tabs.mappings.none"), environments: [] });
                            this.availableTopologies(topologiesArray);
                        };
                        Mappings.prototype._showSelectTopologyDialog = function () {
                            this.selectedTopology(this._activeTopology);
                            this.isSelectTopologyDialogVisible(true);
                        };
                        Mappings.prototype._loadWebApplications = function (environmentId, reload) {
                            var _this = this;
                            if (reload === void 0) { reload = false; }
                            var environmentIds = null;
                            if (environmentId) {
                                environmentIds = [environmentId];
                            }
                            else {
                                this.availableTopologies().forEach(function (topology) {
                                    if (topology.value === _this._activeTopology) {
                                        environmentIds = topology.environments;
                                    }
                                });
                            }
                            var selectedMapping = this._getMapping(this.selectedEnvironment());
                            var listWebApplications = $models.getItem($const.TCMROOT).getListWebApplications(environmentIds, selectedMapping.environmentPurpose);
                            this.webApplicationsLoading(true);
                            if (reload) {
                                listWebApplications.unload();
                            }
                            this._executeOnceWhenItemLoaded(listWebApplications, this.getDelegate(this._onListWebApplicationsLoad), this.getDelegate(this._onListWebApplicationsLoadFailed));
                        };
                        Mappings.prototype._addMapping = function () {
                            this._loadWebApplications(null, true);
                            var selectedMapping = this._getMapping(this.selectedEnvironment());
                            selectedMapping.isEmpty(false);
                            this.notifyPublicationAboutChange();
                        };
                        Mappings.prototype._onListWebApplicationsLoad = function (list) {
                            this._updateWebApplications(list.getId());
                            this.webApplicationsLoading(false);
                        };
                        Mappings.prototype._onListWebApplicationsLoadFailed = function (event) {
                            var selectedMapping = this._getMapping(this.selectedEnvironment());
                            selectedMapping.isConfigurationInvalid(true);
                            this.webApplicationsLoading(false);
                        };
                        Mappings.prototype._updateWebApplications = function (listId) {
                            var listWebApplications = $models.getItem(listId);
                            var xmlDoc = listWebApplications.getXmlDocument();
                            var webApplicationNodes = $xml.selectNodes(xmlDoc, "/pm:WebApplications/pm:WebApplication");
                            var items = Array.prototype.slice.call(webApplicationNodes);
                            var webApplicationsArray = items.map(function (item) { return ({
                                value: $xml.getInnerText(item, "@Id"),
                                text: $xml.getInnerText(item)
                            }); });
                            if (webApplicationsArray.length === 0) {
                                webApplicationsArray.push({ value: null, text: this.localize("tabs.mappings.noneAvailable") });
                                var selectedMapping = this._getMapping(this.selectedEnvironment());
                                selectedMapping.webApplicationId(null);
                            }
                            else {
                                webApplicationsArray.unshift({ value: "", text: this.localize("tabs.mappings.none") });
                            }
                            this.webApplications(webApplicationsArray);
                        };
                        Mappings.prototype._updateData = function () {
                            if (this._mappingsLoadFinished && this._bptsLoadFinished) {
                                this._updateActivePurposes();
                                this._updateTopologyMappings();
                                this._resetListSelection();
                                this.isLoading(false);
                            }
                        };
                        Mappings.prototype._updateActivePurposes = function () {
                            var publication = this._getPublication();
                            var listBusinessProcessTypes = publication.getListBusinessProcessTypes();
                            var purposes = listBusinessProcessTypes.getEnvironmentPurposesForBpt(publication.getBusinessProcessTypeId());
                            this.activeTopologyPurposes(purposes);
                        };
                        Mappings.prototype._updateTopologyMappings = function () {
                            var _this = this;
                            var publication = this._getPublication();
                            var topologyMappingsXml = publication.getTopologyMappingsXml();
                            if (topologyMappingsXml === null) {
                                return;
                            }
                            var topologyMappingsXmlDoc = $xml.getNewXmlDocument(topologyMappingsXml);
                            var mappingNodes = $xml.selectNodes(topologyMappingsXmlDoc, "/pm:Mappings/pm:Mapping");
                            var items = Array.prototype.slice.call(mappingNodes);
                            var mappings = items.map(function (mappingNode) { return _this._getTopologyMapping(mappingNode); });
                            this.activeTopologyPurposes().forEach(function (purpose) {
                                if (!mappings.some(function (mapping) { return mapping.environmentPurpose === purpose; })) {
                                    mappings.push(_this._getEmptyTopologyMapping(purpose));
                                }
                            });
                            this.mappings(mappings);
                        };
                        Mappings.prototype._getEmptyTopologyMapping = function (environmentPurpose) {
                            var _this = this;
                            var result = {
                                id: "",
                                environmentId: "",
                                publicationId: this._publicationId,
                                webApplicationId: ko.observable(""),
                                environmentPurpose: environmentPurpose,
                                baseMappedUrl: ko.observable(""),
                                relativeUrl: ko.observable(""),
                                webSiteUrl: null,
                                isValid: null,
                                hasPublishedItems: ko.observable(false),
                                isEmpty: ko.observable(true),
                                isDirty: ko.observable(false),
                                isActive: ko.observable(true),
                                isDeleted: ko.observable(false),
                                isConfigurationInvalid: ko.observable(this._topologyManagerLoadFailed),
                                relativeUrlErrorText: ko.observable(""),
                                webApplicationErrorText: ko.observable("")
                            };
                            result.isValid = this._isValid(result);
                            result.webSiteUrl = this._getWebSiteUrl(result);
                            result.webApplicationId.subscribe(function (webApplicationId) {
                                var webApplicationsArray = _this.webApplications().filter(function (webApplication) { return webApplication.value === webApplicationId; });
                                if (webApplicationsArray.length > 0) {
                                    result.baseMappedUrl(webApplicationsArray[0].text.replace(/\/$/, ""));
                                }
                            });
                            return result;
                        };
                        Mappings.prototype._getTopologyMapping = function (mapping) {
                            var environmentPurpose = $xml.getInnerText(mapping, "@EnvironmentPurpose");
                            var result = this._getEmptyTopologyMapping(environmentPurpose);
                            result.id = $xml.getInnerText(mapping, "@Id");
                            result.environmentId = $xml.getInnerText(mapping, "@CdEnvironmentId");
                            result.webApplicationId($xml.getInnerText(mapping, "@WebApplicationId"));
                            result.baseMappedUrl($xml.getInnerText(mapping, "@BaseMappedUrl", "").replace(/\/$/, ""));
                            result.relativeUrl(decodeURI($xml.getInnerText(mapping, "@RelativeUrl", "").replace(/^\//, "")));
                            result.isActive(this.activeTopologyPurposes.indexOf(environmentPurpose) !== -1);
                            result.isEmpty(false);
                            result.isDirty(true);
                            result.hasPublishedItems($xml.getInnerText(mapping, "@HasPublishedItems") === "true");
                            return result;
                        };
                        Mappings.prototype._getWebSiteUrl = function (mapping) {
                            var _this = this;
                            return ko.pureComputed(function () {
                                var isWebApplicationIdEmpty = mapping.webApplicationId() == null || mapping.webApplicationId() === "";
                                var baseMappedUrl = !isWebApplicationIdEmpty ? mapping.baseMappedUrl() : "";
                                return _this._combineUrl(baseMappedUrl, mapping.relativeUrl());
                            });
                        };
                        Mappings.prototype._isValid = function (mapping) {
                            var _this = this;
                            return ko.pureComputed(function () {
                                if (mapping.webApplicationId() == null) {
                                    return false;
                                }
                                mapping.webApplicationErrorText("");
                                if (mapping.webApplicationId() === "" || !mapping.baseMappedUrl()) {
                                    mapping.webApplicationErrorText(_this.localize("tabs.mappings.emptyBaseUrl"));
                                    return false;
                                }
                                mapping.relativeUrlErrorText("");
                                var isRelativeUrlValid = _this._isRelativeUrlValid(mapping.relativeUrl());
                                if (!isRelativeUrlValid) {
                                    mapping.relativeUrlErrorText(_this.localize("tabs.mappings.invalidCharacters"));
                                    return false;
                                }
                                var newUrl = _this._combineUrl(mapping.baseMappedUrl(), mapping.relativeUrl());
                                var isUrlAvailable = _this._isUrlAvailable(newUrl, mapping.environmentPurpose);
                                if (!isUrlAvailable) {
                                    mapping.relativeUrlErrorText(_this.localize("tabs.mappings.duplicateValue"));
                                    return false;
                                }
                                return true;
                            });
                        };
                        Mappings.prototype._serializeMapping = function (mapping) {
                            if (!mapping) {
                                return "";
                            }
                            var id = mapping.id ? "Id=\"" + mapping.id + "\"" : "";
                            var webApplicationId = "WebApplicationId=\"" + mapping.webApplicationId() + "\"";
                            var publicationId = "PublicationId=\"" + mapping.publicationId + "\"";
                            var environmentPurpose = "EnvironmentPurpose=\"" + mapping.environmentPurpose + "\"";
                            var baseMappedUrl = "BaseMappedUrl=\"" + mapping.baseMappedUrl() + "\"";
                            var relativeUrl = "RelativeUrl = \"" + encodeURI(mapping.relativeUrl()) + "\"";
                            var cdEnvironmentId = mapping.environmentId ? "CdEnvironmentId=\"" + mapping.environmentId + "\"" : "";
                            var toRemove = mapping.isDeleted() ? "ToRemove=\"true\"" : "";
                            return "<pm:Mapping xmlns:pm=\"" + $const.Namespaces.pm + "\" " + id + " " + webApplicationId + " " + publicationId + " " + environmentPurpose + " " + baseMappedUrl + " " + relativeUrl + "  " + cdEnvironmentId + " " + toRemove + " />";
                        };
                        Mappings.prototype._resetListSelection = function () {
                            if (this.activeTopologyPurposes().length === 0) {
                                this.selectedEnvironment(null);
                                return;
                            }
                            this.selectedEnvironment(this.activeTopologyPurposes()[0]);
                        };
                        Mappings.prototype._updateSelectedBusinessProcessTypeValue = function () {
                            var publication = this._getPublication();
                            var businessProcessTypeId = publication.getBusinessProcessTypeId();
                            this.selectedBusinessProcessType(businessProcessTypeId !== $const.URINULL ? businessProcessTypeId : null);
                        };
                        Mappings.prototype.onMappingTooltipMouseOver = function (mapping, event) {
                            var element = event.target;
                            this.mappingTooltipVisible(true);
                            this.mappingTooltipTargetElement(element);
                        };
                        Mappings.prototype.onMappingTooltipMouseOut = function () {
                            this.mappingTooltipVisible(false);
                        };
                        Mappings.prototype.onEnvironmentPurposeClick = function (environmentPurpose) {
                            this.selectedEnvironment(environmentPurpose);
                        };
                        Mappings.prototype.onAddMappingClick = function () {
                            if (this._activeTopology === "") {
                                this._loadTopologies();
                            }
                            else {
                                this._addMapping();
                            }
                        };
                        Mappings.prototype.onRemoveMappingClick = function () {
                            this.isRemoveMappingDialogVisible(true);
                        };
                        Mappings.prototype.onRemoveMapping = function () {
                            var selectedMapping = this._getMapping(this.selectedEnvironment());
                            selectedMapping.isDeleted(true);
                            this.mappings.replace(selectedMapping, this._getEmptyTopologyMapping(selectedMapping.environmentPurpose));
                            if (selectedMapping.id) {
                                this.mappings.unshift(selectedMapping);
                            }
                            this.notifyPublicationAboutChange();
                            this.isRemoveMappingDialogVisible(false);
                        };
                        Mappings.prototype.onCloseRemoveMappingDialog = function () {
                            this.isRemoveMappingDialogVisible(false);
                        };
                        Mappings.prototype.onChangeTopologyButtonClick = function () {
                            this._loadTopologies();
                        };
                        Mappings.prototype.onSelectTopologyDialogButtonClick = function () {
                            this._activeTopology = this.selectedTopology();
                            this._addMapping();
                            this.isSelectTopologyDialogVisible(false);
                        };
                        Mappings.prototype.onCloseTopologyDialogButtonClick = function () {
                            this.selectedTopology(this._activeTopology);
                            this.isSelectTopologyDialogVisible(false);
                        };
                        Mappings.prototype.onDataChanged = function (data, event) {
                            if ($dom.isIE && event.target !== document.activeElement) {
                                return;
                            }
                            var environment = this.selectedEnvironment();
                            if (environment) {
                                this._getMapping(environment).isDirty(true);
                            }
                            this.notifyPublicationAboutChange();
                        };
                        Mappings.prototype._onPublicationChange = function (event) {
                            if (event.data.context === this) {
                                return;
                            }
                            var xpath = event.data.xpath;
                            if (xpath && xpath.match(/tcm:BusinessProcessType/)) {
                                this._updateSelectedBusinessProcessTypeValue();
                                this._updateData();
                            }
                        };
                        Mappings.prototype._onPublicationCollectData = function (event) {
                            var _this = this;
                            this._setActiveMappingsDirty();
                            var mappingsXml = "";
                            var modifiedMappings = this.mappings().filter(function (mapping) { return mapping.isActive() && !mapping.isEmpty(); });
                            modifiedMappings.forEach(function (mapping) {
                                mappingsXml += _this._serializeMapping(mapping);
                            });
                            var publication = this._getPublication();
                            publication.setTopologyMappingsXml(mappingsXml, this);
                        };
                        Mappings.prototype._onPublicationValidate = function (event) {
                            var mappings = this.mappings();
                            for (var i = 0; i < mappings.length; i++) {
                                if (!mappings[i].isEmpty() && !mappings[i].isValid()) {
                                    $messages.registerError(this.localize("tabs.mappings.unableToSavePublication"), this.localize("tabs.mappings.invalidConfigurationSavePrevented"), null, window, window, false);
                                    return false;
                                }
                            }
                            return true;
                        };
                        Mappings.prototype._onPublicationLoad = function (event) {
                            this._updateSelectedBusinessProcessTypeValue();
                            this._updateData();
                        };
                        Mappings.prototype.isMappingValid = function (environment) {
                            if (environment === void 0) { environment = null; }
                            if (!environment) {
                                environment = this.selectedEnvironment();
                            }
                            return environment ? this._getMapping(environment).isValid() : false;
                        };
                        Mappings.prototype.isMappingDirty = function (environment) {
                            if (environment === void 0) { environment = null; }
                            if (!environment) {
                                environment = this.selectedEnvironment();
                            }
                            return environment ? this._getMapping(environment).isDirty() : false;
                        };
                        Mappings.prototype.isChangeTopologyButtonEnabled = function () {
                            if (this.selectedTopology() === "" || this.availableTopologies().length < 3) {
                                return false;
                            }
                            var currentMapping = this._getMapping(this.selectedEnvironment());
                            return !currentMapping.isEmpty();
                        };
                        Mappings.prototype.notifyPublicationAboutChange = function () {
                            var publication = this._getPublication();
                            publication.setChanged(this);
                        };
                        Mappings.prototype._executeOnceWhenItemLoaded = function (modelObject, loadCallback, loadFailedHandler) {
                            if (modelObject.isLoaded(true)) {
                                loadCallback(modelObject);
                            }
                            else {
                                var handleEvent = function (event) {
                                    removeHandlers(event.source);
                                    loadCallback(event.source);
                                };
                                var handleFailedEvent = function (event) {
                                    removeHandlers(event.source);
                                    if (loadFailedHandler) {
                                        loadFailedHandler(event);
                                    }
                                };
                                var removeHandlers = function (object) {
                                    $evt.removeEventHandler(object, "load", handleEvent);
                                    $evt.removeEventHandler(object, "loadfailed", handleFailedEvent);
                                };
                                $evt.addEventHandler(modelObject, "load", handleEvent);
                                $evt.addEventHandler(modelObject, "loadfailed", handleFailedEvent);
                                modelObject.load(true, $const.OpenMode.VIEW);
                            }
                        };
                        Mappings.prototype._isRelativeUrlValid = function (relativeUrl) {
                            if (relativeUrl === "") {
                                return true;
                            }
                            var regEx = new RegExp("^[a-zA-Z0-9\\.\\-\\_]+([ ]*[a-zA-Z0-9\\.\\-\\_]+)*$");
                            return regEx.test(relativeUrl);
                        };
                        Mappings.prototype._isUrlAvailable = function (url, purpose) {
                            var _this = this;
                            url = url.toLowerCase();
                            return this._existingMappings.every(function (value) {
                                return value.primaryMappedUrl.toLowerCase() !== url
                                    || (value.publicationId === _this._publicationId && value.environmentPurpose === purpose);
                            });
                        };
                        Mappings.prototype._setActiveMappingsDirty = function () {
                            var activeMappings = this.mappings().filter(function (mapping) { return mapping.isActive() && !mapping.isEmpty() && !mapping.isDeleted(); });
                            activeMappings.forEach(function (mapping) {
                                mapping.isDirty(true);
                            });
                        };
                        Mappings.prototype._getMapping = function (environmentPurpose) {
                            var filteredMappings = this.mappings().filter(function (mapping) { return mapping.environmentPurpose === environmentPurpose && !mapping.isDeleted(); });
                            if (filteredMappings.length !== 0) {
                                return filteredMappings[0];
                            }
                            return null;
                        };
                        Mappings.prototype._combineUrl = function (url, relativeUrl) {
                            return url ? url + "/" + encodeURI(relativeUrl) : null;
                        };
                        return Mappings;
                    })(SDL.UI.Core.Knockout.ViewModels.ViewModelBase);
                    ViewModels.Mappings = Mappings;
                    Mappings.prototype.disposeInterface = SDL.Client.Types.OO.nonInheritable(function Mappings$disposeInterface() {
                        var publication = this._getPublication();
                        $evt.removeEventHandler(publication, "change", this.getDelegate(this._onPublicationChange));
                        $evt.removeEventHandler(publication, "collectdata", this.getDelegate(this._onPublicationCollectData));
                        $evt.removeEventHandler(publication, "validate", this.getDelegate(this._onPublicationValidate));
                        $evt.removeEventHandler(publication, "load", this.getDelegate(this._onPublicationLoad));
                    });
                    SDL.Client.Types.OO.createInterface("SDL.Web.UI.PublicationMappings.ViewModels.Mappings", Mappings);
                })(ViewModels = PublicationMappings.ViewModels || (PublicationMappings.ViewModels = {}));
            })(PublicationMappings = UI.PublicationMappings || (UI.PublicationMappings = {}));
        })(UI = Web.UI || (Web.UI = {}));
    })(Web = SDL.Web || (SDL.Web = {}));
})(SDL || (SDL = {}));
