﻿Tridion.Type.registerNamespace("SDL.Web.UI.Models.PublicationMappings");

SDL.Web.UI.Models.PublicationMappings.Publication = function PublicationMappings$Publication(id)
{
	Tridion.OO.enableInterface(this, "SDL.Web.UI.Models.PublicationMappings.Publication");
	this.addInterface("Tridion.ContentManager.Publication", [id]);

	var p = this.properties;
	p.areMappingsAccessible = null;
	p.areMappingsEditable = null;

	p.mappingsListXPath = "/tcm:Publication/tcm:Data/pm:Mappings";
	p.mappingXPath = p.mappingsListXPath + "/pm:Mapping[@EnvironmentPurpose='{0}']";

	this.addExtension("afterLoad", this.getDelegate(this._invalidateTopologyMappingProperties));
};

SDL.Web.UI.Models.PublicationMappings.Publication.prototype.getListBusinessProcessTypes = function PublicationMappings$Publication$getListBusinessProcessTypes(columnFilter)
{
	var filter = new Tridion.ContentManager.ListFilter();
	var conditions = filter.conditions;
	conditions.Recursive = true;
	conditions.ShowNewItems = false;
	filter.columns = columnFilter;
	return this.getList(filter, true, false, "SDL.Web.UI.Models.PublicationMappings.ListExtendedBusinessProcessTypes");
};

SDL.Web.UI.Models.PublicationMappings.Publication.prototype.getTopologyMappingsXml = function PublicationMappings$Publication$getTopologyMappingsXml()
{
	var xmlDoc = this.getXmlDocument();
	if (xmlDoc)
	{
		var node = $xml.selectSingleNode(xmlDoc, "/tcm:Publication/tcm:Data/pm:Mappings");
		if (node)
		{
			return $xml.getOuterXml(node);
		}
	}
};

SDL.Web.UI.Models.PublicationMappings.Publication.prototype.setTopologyMappingsXml = function PublicationMappings$Publication$setTopologyMappingsXml(mappingsXml, context)
{
	var xmlDoc = this.getXmlDocument();
	if (xmlDoc)
	{
		this.setInnerXml("/tcm:Publication/tcm:Data/pm:Mappings", mappingsXml, context);
	}
};

SDL.Web.UI.Models.PublicationMappings.Publication.prototype.areMappingsAccessible = function PublicationMappings$Publication$areMappingsAccessible()
{
	var xmlDoc = this.getXmlDocument();
	if (xmlDoc)
	{
		var p = this.properties;
		if (p.areMappingsAccessible == null)
		{
			p.areMappingsAccessible = !!$xml.selectSingleNode(xmlDoc, p.mappingsListXPath + "[@areMappingsAccessible=\"true\"]");
		}
		return p.areMappingsAccessible;
	}
};

SDL.Web.UI.Models.PublicationMappings.Publication.prototype.areMappingsEditable = function PublicationMappings$Publication$areMappingsEditable()
{
	var xmlDoc = this.getXmlDocument();
	if (xmlDoc)
	{
		var p = this.properties;
		if (p.areMappingsEditable == null)
		{
			p.areMappingsEditable = !!$xml.selectSingleNode(xmlDoc, p.mappingsListXPath + "[@IsEditable=\"true\"]");
		}
		return p.areMappingsEditable;
	}
};

SDL.Web.UI.Models.PublicationMappings.Publication.prototype.hasBptManagementRights = function PublicationMappings$Publication$hasBptManagementRights()
{
	var xmlDoc = this.getXmlDocument();
	if (xmlDoc)
	{
		var bptNode = $xml.selectSingleNode(xmlDoc, "/tcm:Publication/tcm:Data/tcm:BusinessProcessType");
		return bptNode && $xml.getInnerText(bptNode, "@ReadOnly") === "false";
	}
};

SDL.Web.UI.Models.PublicationMappings.Publication.prototype._invalidateTopologyMappingProperties = function PublicationMappings$Publication$_invalidateTopologyMappingProperties()
{
	var p = this.properties;
	p.areMappingsAccessible = null;
};

// ------- marshallableObject inplementations/overrides
SDL.Web.UI.Models.PublicationMappings.Publication.prototype.pack = Tridion.OO.nonInheritable(function PublicationMappings$Publication$pack()
{
	var p = this.properties;
	return {
		areMappingsAccessible: p.areMappingsAccessible,
		areMappingsEditable: p.areMappingsEditable
	};
});

SDL.Web.UI.Models.PublicationMappings.Publication.prototype.unpack = Tridion.OO.nonInheritable(function PublicationMappings$Publication$unpack(data)
{
	if (data)
	{
		var p = this.properties;
		p.areMappingsAccessible = data.AreMappingsAccessible;
		p.areMappingsEditable = data.AreMappingsEditable;
	}
});
// ------- end of marshallableObject overrides