﻿using System.Xml;
using SDL.Web.UI.PublicationMappings.DataExtenders.Extenders;
using Tridion.ContentManager;
using Tridion.Web.UI.Core.Extensibility;
using ItemType = Tridion.ContentManager.ItemType;

namespace SDL.Web.UI.PublicationMappings.DataExtenders
{
    public class DataExtender : Tridion.Web.UI.Core.Extensibility.DataExtender
    {
        public override string Name
        {
            get
            {
                return GetType().FullName;
            }
        }

        public override XmlTextReader ProcessRequest(XmlTextReader reader, PipelineContext context)
        {
            if (reader == null)
            {
                return null;
            }

            var command = (string)context.Parameters["command"];
            var id = GetId(context);
            var itemType = GetItemType(id);

            switch (command)
            {
                case "SaveItem":
                    if (itemType == ItemType.Publication)
                    {
                        var publicationExtender = new PublicationExtender(id);
                        return publicationExtender.ProcessRequest(command, reader, context);
                    }
                    return reader;
                default:
                    return reader;
            }
        }

        public override XmlTextReader ProcessResponse(XmlTextReader reader, PipelineContext context)
        {
            if (reader == null)
            {
                return null;
            }

            var command = (string)context.Parameters["command"];
            var id = GetId(context);
            var itemType = GetItemType(id);

            switch (command)
            {
				case "GetList":
                    var bptListExtender = new BusinessProcessTypeListExtender();
                    return bptListExtender.HandlePostGetList(reader, context);
				case "GetItem":
                case "SaveItem":
                {
                    if (itemType == ItemType.Publication)
                    {
                        var publicationExtender = new PublicationExtender(id);
                        return publicationExtender.ProcessResponse(command, reader, context);
                    }

                    return reader;
                }
                default:
                    return reader;
            }
        }

        public static string GetId(IPipelineContext context)
        {
            object id;
            return context.Parameters.TryGetValue("id", out id) ? (string)id : "";
        }

        public static ItemType GetItemType(string itemId)
        {
            if (!TcmUri.IsValid(itemId))
            {
                return ItemType.None;
            }

            var itemUri = new TcmUri(itemId);

            return itemUri.ItemType;
        }
    }
}