﻿using System.Xml;
using SDL.Web.UI.PublicationMappings.Model.Services.Implementation;
using SDL.Web.UI.PublicationMappings.Model.Services.Interfaces;
using Tridion.ContentManager.CoreService.Client;
using Tridion.Web.CMUtils;
using Tridion.Web.UI.Core.Extensibility;

namespace SDL.Web.UI.PublicationMappings.DataExtenders.Extenders
{
    public class BusinessProcessTypeListExtender
    {
        private readonly ICMSession _cmSession;
        private readonly ITopologyMappings _topologyMappings;

        public BusinessProcessTypeListExtender()
        {
            _cmSession = CMSession.GetInstance();
            _topologyMappings = new TopologyMappings();
        }

        internal BusinessProcessTypeListExtender(ICMSession cmSession, ITopologyMappings topologyMappings)
        {
            _cmSession = cmSession;
            _topologyMappings = topologyMappings;
        }

        public XmlTextReader HandlePostGetList(XmlTextReader reader, PipelineContext context)
        {
            if (reader.Name != "tcm:ListBusinessProcessTypes")
            {
                return reader;
            }

            var xmlDocument = reader.ToXmlDocument();
            var bptNodes = xmlDocument.SelectNodes("/tcm:ListBusinessProcessTypes/tcm:Item", _topologyMappings.XmlNamespaceManager);
            foreach (XmlNode bptNode in bptNodes)
            {
                var bptId = bptNode.Attributes["ID"].Value;
                var bpt = _cmSession.CoreServiceClient.Read(bptId, new ReadOptions()) as BusinessProcessTypeData;

	            var cdTopologyTypeIdAttribute = xmlDocument.CreateAttribute("CdTopologyTypeId");
	            cdTopologyTypeIdAttribute.InnerText = bpt.CdTopologyTypeId;
	            bptNode.Attributes.Append(cdTopologyTypeIdAttribute);

                foreach (TargetTypeData targetType in bpt.TargetTypes)
                {
                    var purposeNode = xmlDocument.CreateNode(XmlNodeType.Element, "pm:Purpose", Model.Constants.PublicationMappingsNamespace.NamespaceName);
                    purposeNode.InnerText = targetType.Purpose;
					bptNode.AppendChild(purposeNode);
                }
            }

            return xmlDocument.ToReader();
        }
    }
}