﻿/// <reference path="../../../../SDL/TypeScript/SDL.Client.UI.Core.Knockout/ViewModels/ViewModelBase.d.ts" />
/// <reference path="../../../../SDL.Web.UI.PublicationMappings.Model/Client/ItemTypes/Publication.d.ts" />
/// <reference path="../../../../SDL.Web.UI.PublicationMappings.Model/Client/Lists/ListCdTopologies.d.ts" />
/// <reference path="../../../../SDL.Web.UI.PublicationMappings.Model/Client/Lists/ListWebApplications.d.ts" />

module SDL.Web.UI.PublicationMappings.ViewModels {

    eval(SDL.Client.Types.OO.enableCustomInheritance);

	declare var $display: any;
	declare var $dom: any;
	declare var $evt: any;
	declare var $xml: any;
    declare var $models: any;
    declare var $const: any;
	declare var $messages: any;
	declare var Tridion: any;

	export interface IMapping {
		publicationId: string;
		environmentPurpose: string;
		primaryMappedUrl: string;
	}
	
	export interface ITopologyMapping {
		id: string;
		publicationId: string;
        environmentId: string;
        environmentPurpose: string;
        webApplicationId: KnockoutObservable<string>;
        relativeUrl: KnockoutObservable<string>;
		baseMappedUrl: KnockoutObservable<string>;
        webSiteUrl: KnockoutComputed<string>;

        hasPublishedItems: KnockoutObservable<boolean>;

        isDeleted: KnockoutObservable<boolean>;
        isEmpty: KnockoutObservable<boolean>;
		isValid: KnockoutComputed<boolean>;
		isActive: KnockoutObservable<boolean>;
		isDirty: KnockoutObservable<boolean>;
		isConfigurationInvalid: KnockoutObservable<boolean>;

		relativeUrlErrorText: KnockoutObservable<string>;
		webApplicationErrorText: KnockoutObservable<string>;
	}

	export interface ITopology {
		text: string;
		value: string;
		environments: Array<string>;
	}

	export interface IDropdownOption {
		text: string;
		value: string;
	}

    export class Mappings extends SDL.UI.Core.Knockout.ViewModels.ViewModelBase {

        private _existingMappings: Array<IMapping> = [];
        private _webApplicationIds: Array<string> = [];

        private _activeTopology: string = "";
        private _publicationId: string = "";
        private _topologyTypeId: string = "";

        private _mappingsLoadFinished: boolean = false;
        private _bptsLoadFinished: boolean = false;

		private _topologyManagerLoadFailed = false;

        public activeTopologyPurposes: KnockoutObservableArray<string> = ko.observableArray([]);
		public mappings: KnockoutObservableArray<ITopologyMapping> = ko.observableArray([]);
        public availableTopologies: KnockoutObservableArray<ITopology> = ko.observableArray([]);
        public businessProcessTypes: KnockoutObservable<Array<IDropdownOption>> = ko.observable<Array<IDropdownOption>>([]);
        public webApplications: KnockoutObservableArray<IDropdownOption> = ko.observableArray([]);

		public mappingTooltipTargetElement: KnockoutObservable<Element> = ko.observable<Element>(null);
        public mappingTooltipVisible: KnockoutObservable<boolean> = ko.observable<boolean>(false);
        public selectedBusinessProcessType: KnockoutObservable<string> = ko.observable<string>(null);
        public selectedTopology: KnockoutObservable<string> = ko.observable<string>(null);
        public selectedEnvironment: KnockoutObservable<string> = ko.observable<string>(null);

        public isConfigurationEditable: KnockoutObservable<boolean> = ko.observable<boolean>(false);
        public isBusinessProcessTypeDropdownDisabled: KnockoutObservable<boolean> = ko.observable<boolean>(true);
        public isSelectTopologyDialogVisible: KnockoutObservable<boolean> = ko.observable<boolean>(false);
        public isRemoveMappingDialogVisible: KnockoutObservable<boolean> = ko.observable<boolean>(false);
        public isTopologyManagerAvailable: KnockoutObservable<boolean> = ko.observable<boolean>(false);
		public isLoading: KnockoutObservable<boolean> = ko.observable<boolean>(false);
        public webApplicationsLoading: KnockoutObservable<boolean> = ko.observable<boolean>(false);

        public topologiesLoadFailed: KnockoutObservable<boolean> = ko.observable<boolean>(false);

		public isEditorVisible: KnockoutComputed<boolean>;

        constructor() {
            super();

            const publication: SDL.Web.UI.Models.PublicationMappings.Publication = $display.getView().getItem();
            this._publicationId = publication.getId();

			this._setupComputedObservables();
			this._setupSubscriptions();

	        this.isLoading(true);

            if (publication.isLoaded()) {
                this._initialize();
            }
            else {
                $evt.addEventHandler(publication, "load", this.getDelegate(this._initialize));
            }

            this._loadBusinessProcessTypes();
        }

		private _setupSubscriptions() {
			this.selectedEnvironment.subscribe((newPurpose) => {

				if (!newPurpose) {
					return;
				}

				this._setActiveMappingsDirty();

				const mapping = this._getMapping(newPurpose);
				if (mapping != null && !mapping.isEmpty()) {
					this._loadWebApplications(mapping.environmentId);
				}
            });

            this.selectedBusinessProcessType.subscribe((newBpt) => {

				this._activeTopology = "";

                const publication = this._getPublication();
                publication.setBusinessProcessType(newBpt, this);

                this._updateData();
            });
		}

		private _setupComputedObservables() {
			this.isEditorVisible = ko.pureComputed(() =>
				this.isTopologyManagerAvailable() && this.selectedBusinessProcessType() && this.businessProcessTypes().length > 0);
		}

		private _getPublication(): SDL.Web.UI.Models.PublicationMappings.Publication {
			return $models.getItem(this._publicationId);
		}

        private _initialize() {

            const publication = this._getPublication();
            $evt.removeEventHandler(publication, "load", this.getDelegate(this._initialize));

            $evt.addEventHandler(publication, "change", this.getDelegate(this._onPublicationChange));
            $evt.addEventHandler(publication, "collectdata", this.getDelegate(this._onPublicationCollectData));
            $evt.addEventHandler(publication, "validate", this.getDelegate(this._onPublicationValidate));
			$evt.addEventHandler(publication, "load", this.getDelegate(this._onPublicationLoad));

			this.isBusinessProcessTypeDropdownDisabled(!publication.hasBptManagementRights());
			this.isTopologyManagerAvailable(publication.areMappingsAccessible());

            this._updateSelectedBusinessProcessTypeValue();

            if (this.isTopologyManagerAvailable()) {
                this._loadTopologyMappings();
			}
			else {
				$messages.registerError(
					this.localize("tabs.mappings.topologyManagerUnavailableTitle"),
					this.localize("tabs.mappings.topologyManagerUnavailableDetails"),
					null, window, window, false);

				this.isLoading(false);
            }
        }


        private _loadBusinessProcessTypes() {
            const publication = this._getPublication();
            const listBusinessProcessTypes = publication.getListBusinessProcessTypes();

            this._executeOnceWhenItemLoaded(
				listBusinessProcessTypes,
				this.getDelegate(this._onListBusinessProcessTypesLoad),
				this.getDelegate(this._onListBusinessProcessTypesLoadFailed));
        }

		private _onListBusinessProcessTypesLoad(list) {
			this._bptsLoadFinished = true;
            this._populateBusinessProcessTypesDropdown();
        }

        private _onListBusinessProcessTypesLoadFailed(event) {
            this.isBusinessProcessTypeDropdownDisabled(true);
        }

        private _populateBusinessProcessTypesDropdown() {
            this._updateListBusinessProcessTypes();
            this._updateData();
        }

        private _updateListBusinessProcessTypes() {
            const publication = this._getPublication();
            const listBusinessProcessTypes = publication.getListBusinessProcessTypes();

            const bptListXmlDocument = listBusinessProcessTypes.getXmlDocument();
            const itemNodes: NodeList = $xml.selectNodes(bptListXmlDocument, "/tcm:ListBusinessProcessTypes/tcm:Item");
			const items: Array<Node> = Array.prototype.slice.call(itemNodes);

			const bptArray = items.map(item => ({
                    value: $xml.getInnerText(item, "@ID"),
                    text: $xml.getInnerText(item, "@Title")
            }));
            bptArray.unshift({ value: null, text: this.localize("tabs.mappings.none") });

            this.businessProcessTypes(bptArray);
        }


		private _loadTopologyMappings() {

            const listTopologyMappings = $models.getItem($const.TCMROOT).getListTopologyMappings();
			listTopologyMappings.unload();
            this._executeOnceWhenItemLoaded(
				listTopologyMappings,
				this.getDelegate(this._onListTopologyMappingsLoad),
				this.getDelegate(this._onListTopologyMappingsLoadFailed));
        }

        private _onListTopologyMappingsLoad(list) {

            const publication = this._getPublication();
			this.isConfigurationEditable(publication.areMappingsEditable());

            this._mappingsLoadFinished = true;
            this._updateExistingTopologyMappings();
        }

        private _onListTopologyMappingsLoadFailed(event) {
        
			this.isConfigurationEditable(false);
			this._topologyManagerLoadFailed = true;

			this._mappingsLoadFinished = true;
            this._updateData();
        }

        private _updateExistingTopologyMappings() {

            const listTopologyMappings = $models.getItem($const.TCMROOT).getListTopologyMappings();

            const xmlDoc: XMLDocument = listTopologyMappings.getXmlDocument();
            const itemsNode: NodeList = $xml.selectNodes(xmlDoc, "/ttm:*/ttm:TopologyMapping");
            const items: Array<Element> = Array.prototype.slice.call(itemsNode);

            this._existingMappings = items.map(item => 
                ({
                    publicationId: item.getAttribute("PublicationId"),
                    environmentPurpose: item.getAttribute("EnvironmentPurpose"),
                    primaryMappedUrl: item.getAttribute("PrimaryMappedUrl")
                })
            );

            this._updateData();
        }


		private _loadTopologies() {

            this.availableTopologies([]);

            const publication = this._getPublication();
            const listBusinessProcessTypes = publication.getListBusinessProcessTypes();

            this._topologyTypeId = listBusinessProcessTypes.getTopologyTypeForBpt(publication.getBusinessProcessTypeId());

            this._webApplicationIds = this.mappings()
				.filter(mapping => !mapping.isDeleted()
					&& !!mapping.webApplicationId()
					&& this.activeTopologyPurposes.indexOf(mapping.environmentPurpose) !== -1)
				.map(mapping => mapping.webApplicationId());

            this.topologiesLoadFailed(false);
            this.isLoading(true);

            const listTopologies: SDL.Web.UI.Models.PublicationMappings.ListCdTopologies = $models.getItem($const.TCMROOT)
				.getListTopologies(this._topologyTypeId, this._webApplicationIds.length > 0 ? this._webApplicationIds : null);
			this._executeOnceWhenItemLoaded(
				listTopologies,
				this.getDelegate(this._onListTopologiesLoad),
				this.getDelegate(this._onListTopologiesLoadFailed));
        }

		private _onListTopologiesLoad(list) {
            this._processListTopologies();
			this.isLoading(false);
        }

        private _onListTopologiesLoadFailed(event) {
			this.topologiesLoadFailed(true);
			this.isLoading(false);
        }

        private _processListTopologies() {

			this._updateTopologies();

			const selectedMapping = this._getMapping(this.selectedEnvironment());
			if (this.availableTopologies().length > 2 || !selectedMapping.isEmpty()) {
				this._showSelectTopologyDialog();
				return;
			}

	        if (this.availableTopologies()[1]) {
		        this.selectedTopology(this.availableTopologies()[1].value);
		        this.onSelectTopologyDialogButtonClick();
			} else {
		        selectedMapping.isConfigurationInvalid(true);
	        }
        }

		private _updateTopologies() {
			const listTopologies: SDL.Web.UI.Models.PublicationMappings.ListCdTopologies = $models.getItem($const.TCMROOT).getListTopologies(this._topologyTypeId, this._webApplicationIds);

            const listTopologiesXmlDoc = listTopologies.getXmlDocument();
            const topologyNodes: NodeList = $xml.selectNodes(listTopologiesXmlDoc, "/pm:CdTopologies/pm:CdTopology");
            const items: Array<Node> = Array.prototype.slice.call(topologyNodes);

            const topologiesArray = items.map(item => {
	            const envNodes: NodeList = $xml.selectNodes(item, "pm:CdEnvironment/@Id");
                const envItems: Array<Node> = Array.prototype.slice.call(envNodes);
	            return {
		            value: $xml.getInnerText(item, "@Id"),
		            text: $xml.getInnerText(item, "@Name"),
		            environments: envItems.map(env => $xml.getInnerText(env))
	            };
            });
            topologiesArray.unshift({ value: "", text: this.localize("tabs.mappings.none"), environments: [] });

            this.availableTopologies(topologiesArray);
		}

		private _showSelectTopologyDialog() {
			this.selectedTopology(this._activeTopology);
            this.isSelectTopologyDialogVisible(true);
		}


		private _loadWebApplications(environmentId: string, reload:boolean = false) {

			let environmentIds = null;

			if (environmentId) {
				environmentIds = [environmentId];
			} else {
				this.availableTopologies().forEach(topology => {
					if (topology.value === this._activeTopology) {
						environmentIds = topology.environments;
					}
				});
			}

            const selectedMapping = this._getMapping(this.selectedEnvironment());
            const listWebApplications: SDL.Web.UI.Models.PublicationMappings.ListWebApplications =
				$models.getItem($const.TCMROOT).getListWebApplications(environmentIds, selectedMapping.environmentPurpose);

			this.webApplicationsLoading(true);

			if (reload) {
				listWebApplications.unload();
			}

			this._executeOnceWhenItemLoaded(
				listWebApplications,
				this.getDelegate(this._onListWebApplicationsLoad),
				this.getDelegate(this._onListWebApplicationsLoadFailed));
		}

		private _addMapping() {
            this._loadWebApplications(null, true);

            const selectedMapping = this._getMapping(this.selectedEnvironment());
			selectedMapping.isEmpty(false);

			this.notifyPublicationAboutChange();
		}

		private _onListWebApplicationsLoad(list) {
            this._updateWebApplications(list.getId());
			this.webApplicationsLoading(false);
		}

        private _onListWebApplicationsLoadFailed(event) {

			const selectedMapping = this._getMapping(this.selectedEnvironment());
			selectedMapping.isConfigurationInvalid(true);

			this.webApplicationsLoading(false);
        }

		private _updateWebApplications(listId: string) {

            const listWebApplications: SDL.Web.UI.Models.PublicationMappings.ListWebApplications = $models.getItem(listId);

			const xmlDoc = listWebApplications.getXmlDocument();
			const webApplicationNodes: NodeList = $xml.selectNodes(xmlDoc, "/pm:WebApplications/pm:WebApplication");
            const items: Array<Node> = Array.prototype.slice.call(webApplicationNodes);

			const webApplicationsArray = items.map(item => ({
				value: $xml.getInnerText(item, "@Id"),
				text: $xml.getInnerText(item)
			}));
			
			if (webApplicationsArray.length === 0) {
				webApplicationsArray.push({ value: null, text: this.localize("tabs.mappings.noneAvailable") });
				const selectedMapping = this._getMapping(this.selectedEnvironment());
				selectedMapping.webApplicationId(null);
			} else {
				webApplicationsArray.unshift({ value: "", text: this.localize("tabs.mappings.none") });
			}

			this.webApplications(webApplicationsArray);
        }
		

        private _updateData() {
            if (this._mappingsLoadFinished && this._bptsLoadFinished) {
                this._updateActivePurposes();
                this._updateTopologyMappings();
                this._resetListSelection();

                this.isLoading(false);
            }
        }
        
		private _updateActivePurposes() {

			const publication = this._getPublication();
            const listBusinessProcessTypes = publication.getListBusinessProcessTypes();
            const purposes = listBusinessProcessTypes.getEnvironmentPurposesForBpt(publication.getBusinessProcessTypeId());

            this.activeTopologyPurposes(purposes);
        }

		private _updateTopologyMappings() {

            const publication = this._getPublication();

			const topologyMappingsXml = publication.getTopologyMappingsXml();

            if (topologyMappingsXml === null) {
                return;
            }

            const topologyMappingsXmlDoc: XMLDocument = $xml.getNewXmlDocument(topologyMappingsXml);
            const mappingNodes: NodeList = $xml.selectNodes(topologyMappingsXmlDoc, "/pm:Mappings/pm:Mapping");
            const items: Array<Element> = Array.prototype.slice.call(mappingNodes);

            const mappings = items.map(mappingNode => this._getTopologyMapping(mappingNode));

            this.activeTopologyPurposes().forEach(purpose => {
                if (!mappings.some(mapping => mapping.environmentPurpose === purpose)) {
					mappings.push(this._getEmptyTopologyMapping(purpose));
                }
            });

			this.mappings(mappings);
		}

		private _getEmptyTopologyMapping(environmentPurpose: string): ITopologyMapping {

			const result: ITopologyMapping = {
				id: "",
				environmentId: "",
				publicationId: this._publicationId,
				webApplicationId: ko.observable<string>(""),
				environmentPurpose,
				baseMappedUrl: ko.observable<string>(""),
				relativeUrl: ko.observable<string>(""),
				webSiteUrl: null,
				isValid: null,
				hasPublishedItems: ko.observable<boolean>(false),
				isEmpty: ko.observable<boolean>(true),
				isDirty: ko.observable<boolean>(false),
				isActive: ko.observable<boolean>(true),
				isDeleted: ko.observable<boolean>(false),
				isConfigurationInvalid: ko.observable<boolean>(this._topologyManagerLoadFailed),
				relativeUrlErrorText: ko.observable<string>(""),
				webApplicationErrorText: ko.observable<string>("")
			};

			result.isValid = this._isValid(result);
			result.webSiteUrl = this._getWebSiteUrl(result);

			result.webApplicationId.subscribe((webApplicationId) => {
				
                const webApplicationsArray = this.webApplications().filter(webApplication => webApplication.value === webApplicationId);
                if (webApplicationsArray.length > 0) {
					result.baseMappedUrl(webApplicationsArray[0].text.replace(/\/$/, ""));
                }
            });

			return result;
		}

		private _getTopologyMapping(mapping: Element): ITopologyMapping {

			const environmentPurpose = $xml.getInnerText(mapping, "@EnvironmentPurpose");

			const result = this._getEmptyTopologyMapping(environmentPurpose);

			result.id = $xml.getInnerText(mapping, "@Id");
			result.environmentId = $xml.getInnerText(mapping, "@CdEnvironmentId");
			result.webApplicationId($xml.getInnerText(mapping, "@WebApplicationId"));
			result.baseMappedUrl($xml.getInnerText(mapping, "@BaseMappedUrl", "").replace(/\/$/, ""));
			result.relativeUrl(decodeURI($xml.getInnerText(mapping, "@RelativeUrl", "").replace(/^\//, "")));
			result.isActive(this.activeTopologyPurposes.indexOf(environmentPurpose) !== -1);
			result.isEmpty(false);
			result.isDirty(true);
			result.hasPublishedItems($xml.getInnerText(mapping, "@HasPublishedItems") === "true");

			return result;
		}

		private _getWebSiteUrl(mapping: ITopologyMapping): KnockoutComputed<string> {
            return ko.pureComputed<string>(() => {
				const isWebApplicationIdEmpty = mapping.webApplicationId() == null || mapping.webApplicationId() === "";
	            const baseMappedUrl = !isWebApplicationIdEmpty ? mapping.baseMappedUrl() : "";
	            return this._combineUrl(baseMappedUrl, mapping.relativeUrl());
            });
        }

		private _isValid(mapping: ITopologyMapping): KnockoutComputed<boolean> {
			return ko.pureComputed<boolean>(() => {

				if (mapping.webApplicationId() == null) {
					return false;
				}

				mapping.webApplicationErrorText("");
				if (mapping.webApplicationId() === "" || !mapping.baseMappedUrl()) {
					mapping.webApplicationErrorText(this.localize("tabs.mappings.emptyBaseUrl"));
					return false;
				}
				
				mapping.relativeUrlErrorText("");
				const isRelativeUrlValid = this._isRelativeUrlValid(mapping.relativeUrl());
				if (!isRelativeUrlValid) {
                    mapping.relativeUrlErrorText(this.localize("tabs.mappings.invalidCharacters"));
					return false;
				}

				const newUrl = this._combineUrl(mapping.baseMappedUrl(), mapping.relativeUrl());
				const isUrlAvailable = this._isUrlAvailable(newUrl, mapping.environmentPurpose);
				if (!isUrlAvailable) {
                    mapping.relativeUrlErrorText(this.localize("tabs.mappings.duplicateValue"));
					return false;
                }

				return true;
			});
		}

		private _serializeMapping(mapping: ITopologyMapping): string {

            if (!mapping) {
                return "";
            }

            const id = mapping.id ? `Id="${mapping.id}"` : ``;
            const webApplicationId = `WebApplicationId="${mapping.webApplicationId()}"`;
            const publicationId = `PublicationId="${mapping.publicationId}"`;
            const environmentPurpose = `EnvironmentPurpose="${mapping.environmentPurpose}"`;
            const baseMappedUrl = `BaseMappedUrl="${mapping.baseMappedUrl()}"`;
            const relativeUrl = `RelativeUrl = "${encodeURI(mapping.relativeUrl())}"`;
            const cdEnvironmentId = mapping.environmentId ? `CdEnvironmentId="${mapping.environmentId}"` : ``;
            const toRemove = mapping.isDeleted() ? `ToRemove="true"` : ``;

            return `<pm:Mapping xmlns:pm="${$const.Namespaces.pm}" ${id} ${webApplicationId} ${publicationId} ${environmentPurpose} ${baseMappedUrl} ${relativeUrl}  ${cdEnvironmentId} ${toRemove} />`;
        }

		private _resetListSelection() {

			if (this.activeTopologyPurposes().length === 0) {
                this.selectedEnvironment(null);
                return;
            }

			this.selectedEnvironment(this.activeTopologyPurposes()[0]);
        }

		private _updateSelectedBusinessProcessTypeValue() {

			const publication = this._getPublication();
            const businessProcessTypeId = publication.getBusinessProcessTypeId();

            this.selectedBusinessProcessType(businessProcessTypeId !== $const.URINULL ? businessProcessTypeId : null);
        }


        public onMappingTooltipMouseOver(mapping, event) {
            const element: Element = event.target;
            this.mappingTooltipVisible(true);
            this.mappingTooltipTargetElement(element);
        }

		public onMappingTooltipMouseOut() {
            this.mappingTooltipVisible(false);
        }

        public onEnvironmentPurposeClick(environmentPurpose) {
            this.selectedEnvironment(environmentPurpose);
        }
        
        public onAddMappingClick() {
			if (this._activeTopology === "") {
				this._loadTopologies();
			} else {
				this._addMapping();
			}
        }

		public onRemoveMappingClick() {
			this.isRemoveMappingDialogVisible(true);
		}

		public onRemoveMapping() {

			const selectedMapping = this._getMapping(this.selectedEnvironment());
			selectedMapping.isDeleted(true);

			this.mappings.replace(selectedMapping, this._getEmptyTopologyMapping(selectedMapping.environmentPurpose));

			if (selectedMapping.id) {
				this.mappings.unshift(selectedMapping);
			}

			this.notifyPublicationAboutChange();

			this.isRemoveMappingDialogVisible(false);
		}

		public onCloseRemoveMappingDialog() {
			this.isRemoveMappingDialogVisible(false);
		}

		public onChangeTopologyButtonClick() {
            this._loadTopologies();
        }

        public onSelectTopologyDialogButtonClick() {
            this._activeTopology = this.selectedTopology();
			this._addMapping();
            this.isSelectTopologyDialogVisible(false);
        }

		public onCloseTopologyDialogButtonClick() {
			this.selectedTopology(this._activeTopology);
            this.isSelectTopologyDialogVisible(false);
        }

		public onDataChanged(data, event) {

			// IE workaround for SE-7304, SE-7301
			// Set of the placeholder attribute triggers input event in IE
			if ($dom.isIE && event.target !== document.activeElement) {
				return;
			}

			const environment = this.selectedEnvironment();
			
			if (environment) {
				this._getMapping(environment).isDirty(true);
			}

			this.notifyPublicationAboutChange();
		}

        private _onPublicationChange(event) {
            if (event.data.context === this) {
                return;
            }

            const xpath: string = event.data.xpath;
            if (xpath && xpath.match(/tcm:BusinessProcessType/)) {
                this._updateSelectedBusinessProcessTypeValue();
                this._updateData();
            }
        }

        private _onPublicationCollectData(event) {

			this._setActiveMappingsDirty();

            let mappingsXml = "";

            const modifiedMappings = this.mappings().filter(mapping => mapping.isActive() && !mapping.isEmpty());
            modifiedMappings.forEach(mapping => {
                mappingsXml += this._serializeMapping(mapping);
            });

            const publication = this._getPublication();
            publication.setTopologyMappingsXml(mappingsXml, this);
        }

        private _onPublicationValidate(event): boolean {

			const mappings = this.mappings();
	        for (let i = 0; i < mappings.length; i++) {
				if (!mappings[i].isEmpty() && !mappings[i].isValid()) {
					$messages.registerError(
						this.localize("tabs.mappings.unableToSavePublication"),
						this.localize("tabs.mappings.invalidConfigurationSavePrevented"),
						null, window, window, false);

					return false;
				}    
	        }

            return true;
        }

        private _onPublicationLoad(event) {
            this._updateSelectedBusinessProcessTypeValue();
            this._updateData();
        }

		public isMappingValid(environment: string = null): boolean {

			if (!environment) {
				environment = this.selectedEnvironment();
			}

			return environment ? this._getMapping(environment).isValid() : false;
		}

		public isMappingDirty(environment: string = null): boolean {

			if (!environment) {
				environment = this.selectedEnvironment();
			}

			return environment ? this._getMapping(environment).isDirty() : false;
		}

		public isChangeTopologyButtonEnabled(): boolean {

            if (this.selectedTopology() === "" || this.availableTopologies().length < 3) {
				return false;
			}

			const currentMapping = this._getMapping(this.selectedEnvironment());
			return !currentMapping.isEmpty();
		}

		public notifyPublicationAboutChange() {
			const publication = this._getPublication();
			publication.setChanged(this);
		}

        private _executeOnceWhenItemLoaded(modelObject: any, loadCallback: Function, loadFailedHandler?: Function) {

            if (modelObject.isLoaded(true)) {
                loadCallback(modelObject);
            }
            else {

                var handleEvent = (event) => {
                    removeHandlers(event.source);
                    loadCallback(event.source);
                };

                var handleFailedEvent = (event) => {
                    removeHandlers(event.source);
                    if (loadFailedHandler) {
                        loadFailedHandler(event);
                    }
                };

                var removeHandlers = (object) => {
                    $evt.removeEventHandler(object, "load", handleEvent);
                    $evt.removeEventHandler(object, "loadfailed", handleFailedEvent);
                };

                $evt.addEventHandler(modelObject, "load", handleEvent);
                $evt.addEventHandler(modelObject, "loadfailed", handleFailedEvent);
                modelObject.load(true, $const.OpenMode.VIEW);
            }
        }

		private _isRelativeUrlValid(relativeUrl: string): boolean {
            if (relativeUrl === "") {
                return true;
            }
            const regEx = new RegExp("^[a-zA-Z0-9\\.\\-\\_]+([ ]*[a-zA-Z0-9\\.\\-\\_]+)*$");
            return regEx.test(relativeUrl);
        }

        private _isUrlAvailable(url: string, purpose: string): boolean {
            url = url.toLowerCase();
            return this._existingMappings.every((value) =>
                value.primaryMappedUrl.toLowerCase() !== url
                || (value.publicationId === this._publicationId && value.environmentPurpose === purpose));
        }

		private _setActiveMappingsDirty() {
			const activeMappings = this.mappings().filter(mapping => mapping.isActive() && !mapping.isEmpty() && !mapping.isDeleted());
            activeMappings.forEach(mapping => {
	            mapping.isDirty(true);
            });
		}

		private _getMapping(environmentPurpose: string): ITopologyMapping {
			const filteredMappings = this.mappings().filter(mapping => mapping.environmentPurpose === environmentPurpose && !mapping.isDeleted());
			if (filteredMappings.length !== 0) {
				return filteredMappings[0];
			}

			return null;
		}

        private _combineUrl(url: string, relativeUrl: string): string {
            return url ? `${url}/${encodeURI(relativeUrl)}` : null;
        }
    }

    Mappings.prototype.disposeInterface = SDL.Client.Types.OO.nonInheritable(function Mappings$disposeInterface() {
        const publication = this._getPublication();
		$evt.removeEventHandler(publication, "change", this.getDelegate(this._onPublicationChange));
        $evt.removeEventHandler(publication, "collectdata", this.getDelegate(this._onPublicationCollectData));
		$evt.removeEventHandler(publication, "validate", this.getDelegate(this._onPublicationValidate));
		$evt.removeEventHandler(publication, "load", this.getDelegate(this._onPublicationLoad));
	});

    SDL.Client.Types.OO.createInterface("SDL.Web.UI.PublicationMappings.ViewModels.Mappings", Mappings);
}