﻿using System;
using System.Linq;
using Tridion.ContentManager.CoreService.Client;
using Tridion.Web.CMUtils;

namespace SDL.Web.UI.PublicationMappings.Model.Extensions
{
    public static class CmSessionExtensions
    {
        internal static Func<ICMSession, string, bool> BptManagmentRightsChecker = 
            (cmSession, publicationId) => HasRights(cmSession, publicationId, Rights.BusinessProcessTypeManagement | Rights.PublicationManagement);

        public static bool HasBptManagmentRights(this ICMSession cmSession, string publicationId)
        {
            return BptManagmentRightsChecker(cmSession, publicationId);
        }

        internal static Func<ICMSession, string, bool> AreMappingsEditableChecker =
            (cmSession, publicationId) => HasRights(cmSession, publicationId, Rights.PublicationManagement);

        public static bool AreMappingsEditable(this ICMSession cmSession, string publicationId)
        {
            return AreMappingsEditableChecker(cmSession, publicationId);
        }

        private static bool HasRights(ICMSession cmSession, string publicationId, Rights rightsToCheck)
        {
            if (cmSession.IsAdministrator)
            {
                return true;
            }

            var securityDescriptor = cmSession.CoreServiceClient
                 .GetSecurityDescriptorsForSubjectsIds(new[] { publicationId })
                 .First().Value;

            return HasRight(securityDescriptor.Rights, rightsToCheck);
        }

        private static bool HasRight(Rights allow, Rights right)
        {
            return (allow & right) == right;
        }
    }
}
