﻿using System;
using System.Collections.Generic;
using System.Net;
using ImpromptuInterface;
using Microsoft.OData.Client;
using Tridion.TopologyManager.Client;

namespace SDL.Web.UI.PublicationMappings.Model.Services.Interfaces
{
    public interface ITopologyManagerClient
    {
        IEnumerable<MappingData> Mappings { get; }
        IEnumerable<CdTopologyData> CdTopologies { get; }
        IEnumerable<WebApplicationData> WebApplications { get; }
        IEnumerable<WebsiteData> Websites { get; }
        void AddToMappings(MappingData data);
        DataServiceResponse SaveChanges();
        DataServiceResponse SaveChanges(SaveChangesOptions options);
        void UpdateObject(object entity);
	    void DeleteObject(object entity);
    }

    public class TopologyManagerClientFactory
    {
        public ITopologyManagerClient Create()
        {
            var tmClient = new TopologyManagerClient { Credentials = CredentialCache.DefaultCredentials };
            return tmClient.ActLike<ITopologyManagerClient>();
        }
    }

    public static class TopologyMappingQueryExtensions
    {
        internal static Func<IEnumerable<MappingData>, string, IEnumerable<MappingData>> MappingsExpander =
            (data, path) => ((DataServiceQuery<MappingData>)data).Expand(path);

        public static IEnumerable<MappingData> Expand(this IEnumerable<MappingData> data, string path)
        {
            return MappingsExpander(data, path);
        }

        internal static Func<IEnumerable<WebApplicationData>, string, IEnumerable<WebApplicationData>> WebApplicationExpander =
            (data, path) => ((DataServiceQuery<WebApplicationData>)data).Expand(path);

        public static IEnumerable<WebApplicationData> Expand(this IEnumerable<WebApplicationData> data, string path)
        {
            return WebApplicationExpander(data, path);
        }
    }
}