﻿using System.Xml;
using System.Xml.Linq;
using NSubstitute;
using NUnit.Framework;

namespace SDl.Web.UI.PublicationMappings.Tests.DataExtenders.Publication
{
    [TestFixture]
    public class PostSavePublicationFixture : ProcessResponcePublicationFixtureBase
    {
        private const string MappingsXml = 
            "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\">" +
                "<pm:Mapping Id=\"Mapping_id_1\" WebApplicationId=\"web_id_1\" RelativeUrl=\"/test\" />" +
                "<pm:Mapping Id=\"Mapping_id_2\" WebApplicationId=\"web_id_2\" RelativeUrl=\"/new\" />" +
                "<pm:Mapping Id=\"Other_Mapping_id\" WebApplicationId=\"other_id\" RelativeUrl=\"/old\" />" +
            "</pm:Mappings>";

        protected override string GetCommand()
        {
            return "SaveItem";
        }

        [Test]
        public void Should_take_mappings_from_context_and_save_them()
        {
			const string mappingsXmlString =
					   "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\">" +
						   "<pm:Mapping WebApplicationId=\"Test web app\" EnvironmentPurpose=\"QA\" BaseMappedUrl=\"http://qa.test\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" />" +
						   "<pm:Mapping WebApplicationId=\"Live web app\" EnvironmentPurpose=\"Live\" BaseMappedUrl=\"http://live.net\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" />" +
						   "<pm:Mapping WebApplicationId=\"Staging web app\" EnvironmentPurpose=\"Staging\" BaseMappedUrl=\"http://staging.test\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" CdEnvironmentId=\"SomeCdEnvironmentId\" />" +
					   "</pm:Mappings>";

			var mappingsXml = XElement.Parse(mappingsXmlString);
			TopologyMappings.GetMappingsXml(PublicationId).Returns(mappingsXml);

			Context.Parameters.Add("pm:mappings", MappingsXml);

            Extender.ProcessResponse(GetCommand(), Reader, Context);

            TopologyMappings.Received().UpdateMappings(Arg.Is<XmlDocument>(x=>x.OuterXml == MappingsXml));
        }

        [Test]
        public void Should_read_mappings_after_saving()
        {
            const string mappingsXmlString =
                       "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\">" +
                           "<pm:Mapping WebApplicationId=\"Test web app\" EnvironmentPurpose=\"QA\" BaseMappedUrl=\"http://qa.test\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" />" +
                           "<pm:Mapping WebApplicationId=\"Live web app\" EnvironmentPurpose=\"Live\" BaseMappedUrl=\"http://live.net\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" />" +
                           "<pm:Mapping WebApplicationId=\"Staging web app\" EnvironmentPurpose=\"Staging\" BaseMappedUrl=\"http://staging.test\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" CdEnvironmentId=\"SomeCdEnvironmentId\" />" +
                       "</pm:Mappings>";

            var mappingsXml = XElement.Parse(mappingsXmlString);
            TopologyMappings.GetMappingsXml(PublicationId).Returns(mappingsXml);

            Context.Parameters.Add("pm:mappings", MappingsXml);

            Extender.ProcessResponse(GetCommand(), Reader, Context);

            Received.InOrder(() =>
            {
                TopologyMappings.Received().UpdateMappings(Arg.Any<XmlDocument>());
                TopologyMappings.Received().GetMappingsXml(Arg.Any<string>());
            });
        }

        [Test]
        public void Should_not_call_UpdateMappings_if_no_context_data()
        {
			const string mappingsXmlString =
				   "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\">" +
					   "<pm:Mapping WebApplicationId=\"Test web app\" EnvironmentPurpose=\"QA\" BaseMappedUrl=\"http://qa.test\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" />" +
					   "<pm:Mapping WebApplicationId=\"Live web app\" EnvironmentPurpose=\"Live\" BaseMappedUrl=\"http://live.net\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" />" +
					   "<pm:Mapping WebApplicationId=\"Staging web app\" EnvironmentPurpose=\"Staging\" BaseMappedUrl=\"http://staging.test\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" CdEnvironmentId=\"SomeCdEnvironmentId\" />" +
				   "</pm:Mappings>";

			var mappingsXml = XElement.Parse(mappingsXmlString);
			TopologyMappings.GetMappingsXml(PublicationId).Returns(mappingsXml);

            Extender.ProcessResponse(GetCommand(), Reader, Context);

            TopologyMappings.DidNotReceive().UpdateMappings(Arg.Any<XmlDocument>());
        }
    }
}
