using System.Xml;
using System.Xml.Linq;
using NSubstitute;
using NUnit.Framework;
using SDL.Web.UI.PublicationMappings.DataExtenders;
using Tridion.ContentManager;
using Tridion.Web.UI.Core.Extensibility;

namespace SDl.Web.UI.PublicationMappings.Tests.DataExtenders.Publication
{
    public abstract class ProcessResponcePublicationFixtureBase : PublicationExtenderFixtureBase
    {
        protected PipelineContext Context;
        protected XmlTextReader Reader;

        [SetUp]
        public new void SetUp()
        {
            base.SetUp();

            Reader = DataExtenderTestsHelper.GetReader(GetInputXmlString());
            Context = DataExtenderTestsHelper.GetPipelineContext(GetCommand(), PublicationId);
        }

        protected string GetInputXmlString()
        {
            return "<tcm:Publication ID=\"tcm:0-5-1\" xmlns:tcm=\"http://www.tridion.com/ContentManager/5.0\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\">" +
                        "<tcm:Info />" +
                        "<tcm:Data>" +
                            "<tcm:BusinessProcessType xlink:type=\"simple\" xlink:title=\"StagingLiveBusinessProcessType\" xlink:href=\"tcm:1-15-4096\" />" +
                        "</tcm:Data>" +
                        "<tcm:Rights />" +
                    "</tcm:Publication>";
        }
        protected abstract string GetCommand();

        [Test]
        [TestCase(true)]
        [TestCase(false)]
        public void Should_set_bpt_read_only_attribute(bool readOnly)
        {

            const string mappingsXmlString = "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\" />";

            var mappingsXml = XElement.Parse(mappingsXmlString);
            TopologyMappings.GetMappingsXml(PublicationId).Returns(mappingsXml);

            MockRights(!readOnly);
            MockBptId(PublicationId, TcmUri.UriNull);

            var result = Extender.ProcessResponse(GetCommand(), Reader, Context);

            var expected =
                "<tcm:Publication ID=\"tcm:0-5-1\" xmlns:tcm=\"http://www.tridion.com/ContentManager/5.0\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\">" +
                    "<tcm:Info />" +
                        "<tcm:Data>" +
                        "<tcm:BusinessProcessType xlink:type=\"simple\" xlink:title=\"StagingLiveBusinessProcessType\" xlink:href=\"tcm:1-15-4096\" ReadOnly=\"" + readOnly.ToString().ToLower() + "\" />" +
                            "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\" IsEditable=\"true\" />" +
                        "</tcm:Data>" +
                    "<tcm:Rights />" +
                "</tcm:Publication>";


            Assert.AreEqual(expected, result.ToXmlDocument().InnerXml);
        }

        [Test]
        [TestCase(true)]
        [TestCase(false)]
        public void Should_add_mappings_and_set_editable_flag(bool editable)
        {
            const string mappingsXmlString =
                "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\">" +
                    "<pm:Mapping WebApplicationId=\"Test web app\" EnvironmentPurpose=\"QA\" BaseMappedUrl=\"http://qa.test\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" />" +
                    "<pm:Mapping WebApplicationId=\"Live web app\" EnvironmentPurpose=\"Live\" BaseMappedUrl=\"http://live.net\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" />" +
                    "<pm:Mapping WebApplicationId=\"Staging web app\" EnvironmentPurpose=\"Staging\" BaseMappedUrl=\"http://staging.test\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" CdEnvironmentId=\"SomeCdEnvironmentId\" />" +
                "</pm:Mappings>";

            var mappingsXml = XElement.Parse(mappingsXmlString);
            TopologyMappings.GetMappingsXml(PublicationId).Returns(mappingsXml);

            MockRights(false, editable);
            MockBptId(PublicationId, TcmUri.UriNull);

            var result = Extender.ProcessResponse(GetCommand(), Reader, Context);

            var expected =
                "<tcm:Publication ID=\"tcm:0-5-1\" xmlns:tcm=\"http://www.tridion.com/ContentManager/5.0\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\">" +
                    "<tcm:Info />" +
                    "<tcm:Data>" +
                    "<tcm:BusinessProcessType xlink:type=\"simple\" xlink:title=\"StagingLiveBusinessProcessType\" xlink:href=\"tcm:1-15-4096\" ReadOnly=\"true\" />" +
                    "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\" IsEditable=\"" + editable.ToString().ToLower() + "\">" +
                        "<pm:Mapping WebApplicationId=\"Test web app\" EnvironmentPurpose=\"QA\" BaseMappedUrl=\"http://qa.test\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" />" +
                        "<pm:Mapping WebApplicationId=\"Live web app\" EnvironmentPurpose=\"Live\" BaseMappedUrl=\"http://live.net\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" />" +
                        "<pm:Mapping WebApplicationId=\"Staging web app\" EnvironmentPurpose=\"Staging\" BaseMappedUrl=\"http://staging.test\" RelativeUrl=\"/\" PublicationId=\"tcm:0-6-1\" CdEnvironmentId=\"SomeCdEnvironmentId\" />" +
                    "</pm:Mappings>" +
                    "</tcm:Data>" +
                    "<tcm:Rights />" +
                "</tcm:Publication>";


            Assert.AreEqual(expected, result.ToXmlDocument().InnerXml);
        }
    }
}