using System;
using NSubstitute;
using NUnit.Framework;
using SDL.Web.UI.PublicationMappings.DataExtenders.Extenders;
using SDL.Web.UI.PublicationMappings.Model.Extensions;
using SDL.Web.UI.PublicationMappings.Model.Services.Interfaces;
using Tridion.ContentManager;
using Tridion.ContentManager.CoreService.Client;
using Tridion.Web.CMUtils;

namespace SDl.Web.UI.PublicationMappings.Tests.DataExtenders.Publication
{
    public class PublicationExtenderFixtureBase
    {
        protected PublicationExtender Extender;
        protected ITopologyMappings TopologyMappings;

        protected ICMSession CmSession;
        protected ISessionAwareCoreService CoreService;
        protected const string PublicationId = "tcm:0-5-1";
        protected static readonly TcmUri BptId = new TcmUri("tcm:6-4-4096");

        private Func<ICMSession, string, bool> _currentBptRightsChecker;
        private Func<ICMSession, string, bool> _currentEditableChecker;

        [TestFixtureSetUp]
        public void TestFixtureSetUp()
        {
            _currentBptRightsChecker = CmSessionExtensions.BptManagmentRightsChecker;
            _currentEditableChecker = CmSessionExtensions.AreMappingsEditableChecker;
        }

        [TestFixtureTearDown]
        public void TestFixtureTearDown()
        {
            CmSessionExtensions.BptManagmentRightsChecker = _currentBptRightsChecker;
            CmSessionExtensions.AreMappingsEditableChecker = _currentEditableChecker;
        }

        [SetUp]
        public void SetUp()
        {
            CmSession = Substitute.For<ICMSession>();
            CoreService = Substitute.For<ISessionAwareCoreService>();

            CmSession.CoreServiceClient.Returns(CoreService);

            TopologyMappings = Substitute.For<ITopologyMappings>();

            TopologyMappings.XmlNamespaceManager.Returns(DataExtenderTestsHelper.XmlNamespaceManager);

            Extender = new PublicationExtender(PublicationId, CmSession, TopologyMappings);

            MockRights();
        }

        protected static void MockRights(bool bptManagementAllowed = true, bool editable = true)
        {
            CmSessionExtensions.BptManagmentRightsChecker = (session, publicationId) => bptManagementAllowed;
            CmSessionExtensions.AreMappingsEditableChecker = (session, publicationId) => editable;
        }

        protected void MockBptId(string publicationId, TcmUri bptId)
        {
            var publicationData = new PublicationData
            {
                BusinessProcessType = new LinkToBusinessProcessTypeData { IdRef = bptId }
            };
            CoreService.Read(publicationId, Arg.Any<ReadOptions>()).Returns(publicationData);
        }
    }
}