﻿using NUnit.Framework;
using SDL.Web.UI.PublicationMappings.DataExtenders;
using Tridion.ContentManager;
using DataExtender = SDL.Web.UI.PublicationMappings.DataExtenders.DataExtender;

namespace SDl.Web.UI.PublicationMappings.Tests.DataExtenders
{
    [TestFixture]
    public class PublicationMappingsDataExtenderFixture
    {
        [SetUp]
        public void SetUp()
        {
        }

        [Test]
        public void Should_return_correct_name()
        {
            Assert.AreEqual(new DataExtender().Name, "SDL.Web.UI.PublicationMappings.DataExtenders.DataExtender");
        }

        protected string GetInputXmlString()
        {
            return "<tcm:Publication ID=\"tcm:0-5-1\" xmlns:tcm=\"http://www.tridion.com/ContentManager/5.0\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\">" +
                        "<tcm:Info />" +
                        "<tcm:Data>" +
                            "<tcm:BusinessProcessType xlink:type=\"simple\" xlink:title=\"StagingLiveBusinessProcessType\" xlink:href=\"tcm:1-15-4096\" />" +
                        "</tcm:Data>" +
                        "<tcm:Rights />" +
                    "</tcm:Publication>";
        }

        [Test]
        public void Should_not_chang_xml_if_wrong_command()
        {
            var input = GetInputXmlString();
            var reader = DataExtenderTestsHelper.GetReader(input);
            var context = DataExtenderTestsHelper.GetPipelineContext("SomeOtherCommand");

            var result = new DataExtender().ProcessResponse(reader, context);

            Assert.AreEqual(input, result.ToXmlDocument().InnerXml);
        }

        [Test]
        [TestCase("tcm:0-1-1", ItemType.Publication)]
        [TestCase("tcm:0-1-4096", ItemType.BusinessProcessType)]
        [TestCase("tcm:0-1-64", ItemType.Page)]
        [TestCase("tcm:0-0-0", ItemType.None)]
        [TestCase("", ItemType.None)]
        [TestCase(null, ItemType.None)]
        [TestCase("incorrect_string", ItemType.None)]
        public void Should_return_correct_item_type(string id, ItemType expectedType)
        {
            var actual = DataExtender.GetItemType(id);
            Assert.AreEqual(actual, expectedType);
        }

        [Test]
        [TestCase("tcm:0-1-1", "tcm:0-1-1")]
        [TestCase("", "")]
        [TestCase(null, "")]
        public void Should_take_id_from_context(string id, string expected)
        {
            var context = DataExtenderTestsHelper.GetPipelineContext("command", id);

            var actual = DataExtender.GetId(context);

            Assert.AreEqual(actual, expected);
        }
    }
}
