﻿using System.IO;
using System.Xml;
using System.Xml.Linq;
using NUnit.Framework;
using SDL.Web.UI.PublicationMappings.DataExtenders;

namespace SDl.Web.UI.PublicationMappings.Tests.Extensions
{
    [TestFixture]
    public class XmlExtensionsFixture
    {
        private const string XmlString =
            "<tcm:Publication ID=\"tcm:0-5-1\" xmlns:tcm=\"http://www.tridion.com/ContentManager/5.0\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\">" +
                "<tcm:Info />" +
                "<tcm:Data />" +
                "<tcm:Rights>" +
                    "<tcm:RightInfo />" +
                    "<tcm:OtherNodes />" +
                    "<pm:HasPublicationMappingsRights xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\">1</pm:HasPublicationMappingsRights>" +
                "</tcm:Rights>" +
            "</tcm:Publication>";

        [Test]
        public void Should_convert_xml_document_to_reader()
        {
            var document = new XmlDocument();
            document.LoadXml(XmlString);

            var actual = document.ToReader();

            Assert.IsTrue(actual is XmlTextReader);
            Assert.AreEqual(XmlString, actual.ReadOuterXml());
        }

        [Test]
        public void Should_convert_reader_to_xml_document()
        {
            var stream = new StringReader(XmlString);
            var reader = new XmlTextReader(stream);

            var actual = reader.ToXmlDocument();

            Assert.IsTrue(actual is XmlDocument);
            Assert.AreEqual(XmlString, actual.OuterXml);
        }


        [Test]
        public void Should_convert_x_element_to_xml_element()
        {
            var xElement = XElement.Parse(XmlString);

            var actual = xElement.ToXmlElement();

            Assert.IsTrue(actual is XmlElement);
            Assert.AreEqual(XmlString, actual.OuterXml);
        }
    }
}
