using System;
using System.Collections.Generic;
using NSubstitute;
using NUnit.Framework;
using SDL.Web.UI.PublicationMappings.Model.Extensions;
using SDL.Web.UI.PublicationMappings.Model.Services.Interfaces;
using Tridion.ContentManager.CoreService.Client;
using Tridion.TopologyManager.Client;
using Tridion.Web.CMUtils;

namespace SDl.Web.UI.PublicationMappings.Tests.Services.TopologyMappings
{
    public class TopologyMappingsFixtureBase
    {
        protected const string PublicationId = "tcm:0-6-1";
        protected ITopologyManagerClient TopologyManagerClient;
        protected ISessionAwareCoreService CoreService;
        protected ICMSession CmSession;

        protected SDL.Web.UI.PublicationMappings.Model.Services.Implementation.TopologyMappings TopologyMappingsService;
        private Func<ICMSession, string, bool> _currentBptRightsChecker;
        private Func<IEnumerable<MappingData>, string, IEnumerable<MappingData>> _currentMappingsExpander;

        [TestFixtureSetUp]
        public void TestFixtureSetUp()
        {
            _currentMappingsExpander = TopologyMappingQueryExtensions.MappingsExpander;
            _currentBptRightsChecker = CmSessionExtensions.BptManagmentRightsChecker;
        }

        [TestFixtureTearDown]
        public void TestFixtureTearDown()
        {
            TopologyMappingQueryExtensions.MappingsExpander = _currentMappingsExpander;
            CmSessionExtensions.BptManagmentRightsChecker = _currentBptRightsChecker;
        }

        [SetUp]
        public void SetUp()
        {
            CmSession = Substitute.For<ICMSession>();
            TopologyManagerClient = Substitute.For<ITopologyManagerClient>();
	        CoreService = Substitute.For<ISessionAwareCoreService>();

			TopologyMappingsService = new SDL.Web.UI.PublicationMappings.Model.Services.Implementation.TopologyMappings(TopologyManagerClient, CoreService);
        }

        protected static void MockRights(bool allowed = true)
        {
            CmSessionExtensions.BptManagmentRightsChecker = (session, publicationId) => allowed;
        }
    }
}