﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using Microsoft.OData.Client;
using NSubstitute;
using NSubstitute.ExceptionExtensions;
using NUnit.Framework;
using Tridion.TopologyManager.Client;

namespace SDl.Web.UI.PublicationMappings.Tests.Services.TopologyMappings
{
    [TestFixture]
    public class UpdateMappingsFixture : TopologyMappingsFixtureBase
    {
        private readonly IEnumerable<MappingData> _mappings = new[]
		{
            new MappingData { Id = "Mapping_id_1", RelativeUrl = "/test", WebApplicationId = "web_id_1", EnvironmentPurpose = "Live", PrimaryMappedUrl = "http://localhost:81/", PublicationId = "tcm:0-12-1"},
            new MappingData { Id = "Mapping_id_2", RelativeUrl = "/new", WebApplicationId = "web_id_2", EnvironmentPurpose = "Draft", PrimaryMappedUrl = "http://localhost:82/", PublicationId = "tcm:0-17-1" },
            new MappingData { Id = "Other_Mapping_id", RelativeUrl = "/old", WebApplicationId = "other_id", EnvironmentPurpose = "Staging", PrimaryMappedUrl = "http://sdl.com/", PublicationId = "tcm:0-12234-1" }
        };

        private static XmlDocument GetMappingsXml(IEnumerable<MappingData> mappings)
        {
            var xmlString = "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\">";

            foreach (var mapping in mappings)
            {
				xmlString += "<pm:Mapping Id=\"" + mapping.Id
							 + "\" WebApplicationId=\"" + mapping.WebApplicationId
							 + "\" RelativeUrl=\"" + mapping.RelativeUrl
							 + "\" EnvironmentPurpose=\"" + mapping.EnvironmentPurpose
							 + "\" PublicationId=\"" + mapping.PublicationId
							 + "\" BaseMappedUrl=\"" + mapping.PrimaryMappedUrl
							 + "\"/>";
			}

			xmlString += "</pm:Mappings>";

            var result = new XmlDocument();
            result.LoadXml(xmlString);

            return result;
        }

		private static XmlDocument GetMappingsXmlWithFirstItemToRemove(IEnumerable<MappingData> mappings)
		{
			var xmlString = "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\">";

			var mappingsList = mappings.ToList();

			for (var i = 0; i < mappingsList.Count; i++)
			{
				var mapping = mappingsList[i];

				if (i == 0)
				{
					xmlString += "<pm:Mapping Id=\"" + mapping.Id + "\" ToRemove=\"True\" />";
				}
				else
				{
					xmlString += "<pm:Mapping Id=\"" + mapping.Id
					             + "\" WebApplicationId=\"" + mapping.WebApplicationId
					             + "\" RelativeUrl=\"" + mapping.RelativeUrl
								 + "\" EnvironmentPurpose=\"" + mapping.EnvironmentPurpose
								 + "\" PublicationId=\"" + mapping.PublicationId
								 + "\" BaseMappedUrl=\"" + mapping.PrimaryMappedUrl
								 + "\"/>";
				}
			}

			xmlString += "</pm:Mappings>";

			var result = new XmlDocument();
			result.LoadXml(xmlString);

			return result;
		}

		private static XmlDocument GetMappingsXmlWithFirstItemToRemoveAndSecondWithFakeId(IEnumerable<MappingData> mappings)
		{
			var xmlString = "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\">";

			var mappingsList = mappings.ToList();

			for (var i = 0; i < mappingsList.Count; i++)
			{
				var mapping = mappingsList[i];

				if (i == 0)
				{
					xmlString += "<pm:Mapping Id=\"" + mapping.Id + "\" ToRemove=\"True\" />";
				}
				else if (i == 1)
				{
					xmlString += "<pm:Mapping Id=\"FakeId\" WebApplicationId=\"" + mapping.WebApplicationId
								 + "\" RelativeUrl=\"" + mapping.RelativeUrl
								 + "\" EnvironmentPurpose=\"" + mapping.EnvironmentPurpose
								 + "\" PublicationId=\"" + mapping.PublicationId
								 + "\" BaseMappedUrl=\"" + mapping.PrimaryMappedUrl
								 + "\"/>";

				}
				else
				{
					xmlString += "<pm:Mapping Id=\"" + mapping.Id
								 + "\" WebApplicationId=\"" + mapping.WebApplicationId
								 + "\" RelativeUrl=\"" + mapping.RelativeUrl
								 + "\" EnvironmentPurpose=\"" + mapping.EnvironmentPurpose
								 + "\" PublicationId=\"" + mapping.PublicationId
								 + "\" BaseMappedUrl=\"" + mapping.PrimaryMappedUrl
								 + "\"/>";
				}
			}

			xmlString += "</pm:Mappings>";

			var result = new XmlDocument();
			result.LoadXml(xmlString);

			return result;
		}

	    private static XmlDocument GetMappingsXmlWithEmptyId(IEnumerable<MappingData> mappings)
	    {
			var xmlString = "<pm:Mappings xmlns:pm=\"http://www.sdltridion.com/2015/PublicationMappings\">";

			var mappingsList = mappings.ToList();

			for (var i = 0; i < mappingsList.Count; i++)
			{
				var mapping = mappingsList[i];

				if (i == 0)
				{
					xmlString += "<pm:Mapping Id=\"\" WebApplicationId=\"" + mapping.WebApplicationId
						+ "\" RelativeUrl=\"" + mapping.RelativeUrl
						+ "\" EnvironmentPurpose=\"" + mapping.EnvironmentPurpose
						+ "\" PublicationId=\"" + mapping.PublicationId
						+ "\" BaseMappedUrl=\"" + mapping.PrimaryMappedUrl
						+ "\"/>";
				}
				else
				{
					xmlString += "<pm:Mapping Id=\"" + mapping.Id
								 + "\" WebApplicationId=\"" + mapping.WebApplicationId
								 + "\" RelativeUrl=\"" + mapping.RelativeUrl
								 + "\" EnvironmentPurpose=\"" + mapping.EnvironmentPurpose
								 + "\" PublicationId=\"" + mapping.PublicationId
								 + "\" BaseMappedUrl=\"" + mapping.PrimaryMappedUrl
								 + "\"/>";
				}
			}

			xmlString += "</pm:Mappings>";

			var result = new XmlDocument();
			result.LoadXml(xmlString);

			return result;
		}


		[Test]
        public void Should_save_mappings()
        {
            TopologyManagerClient.Mappings.Returns(_mappings);

            var mappingsXml = GetMappingsXml(_mappings);
            TopologyMappingsService.UpdateMappings(mappingsXml);

            Received.InOrder(() =>
            {
                TopologyManagerClient.UpdateObject(ArgIs(_mappings.ElementAt(0)));
                TopologyManagerClient.UpdateObject(ArgIs(_mappings.ElementAt(1)));
                TopologyManagerClient.UpdateObject(ArgIs(_mappings.ElementAt(2)));
                TopologyManagerClient.SaveChanges(SaveChangesOptions.BatchWithSingleChangeset);
            });
        }

	    [Test]
	    public void Should_remove_first_and_update_rest()
	    {
			TopologyManagerClient.Mappings.Returns(_mappings);

			var mappingsXml = GetMappingsXmlWithFirstItemToRemove(_mappings);
			TopologyMappingsService.UpdateMappings(mappingsXml);

			Received.InOrder(() =>
			{
				TopologyManagerClient.DeleteObject(ArgIs(_mappings.ElementAt(0)));
				TopologyManagerClient.UpdateObject(ArgIs(_mappings.ElementAt(1)));
				TopologyManagerClient.UpdateObject(ArgIs(_mappings.ElementAt(2)));
				TopologyManagerClient.SaveChanges(SaveChangesOptions.BatchWithSingleChangeset);
			});
		}

		[Test]
		public void Should_remove_first_skip_fake_and_update_rest()
		{
			TopologyManagerClient.Mappings.Returns(_mappings);

			var mappingsXml = GetMappingsXmlWithFirstItemToRemoveAndSecondWithFakeId(_mappings);
			TopologyMappingsService.UpdateMappings(mappingsXml);

			Received.InOrder(() =>
			{
				TopologyManagerClient.DeleteObject(ArgIs(_mappings.ElementAt(0)));
				TopologyManagerClient.UpdateObject(ArgIs(_mappings.ElementAt(2)));
				TopologyManagerClient.SaveChanges(SaveChangesOptions.BatchWithSingleChangeset);
			});
		}

	    [Test]
	    public void Should_add_new_mapping_if_empty_id()
	    {
			TopologyManagerClient.Mappings.Returns(_mappings);

			var mappingsXml = GetMappingsXmlWithEmptyId(_mappings);
			TopologyMappingsService.UpdateMappings(mappingsXml);
			
			Received.InOrder(() =>
			{
				TopologyManagerClient.AddToMappings(Arg.Is<MappingData>(x =>
					x.Id == String.Empty
					&& x.WebApplicationId == _mappings.ElementAt(0).WebApplicationId
					&& x.RelativeUrl == _mappings.ElementAt(0).RelativeUrl));
				TopologyManagerClient.UpdateObject(ArgIs(_mappings.ElementAt(1)));
				TopologyManagerClient.UpdateObject(ArgIs(_mappings.ElementAt(2)));
				TopologyManagerClient.SaveChanges(SaveChangesOptions.BatchWithSingleChangeset);
			});
		}

		[Test]
        public void Should_ignore_not_existing_mappings()
        {
            TopologyManagerClient.Mappings.Returns(new[]
            {
                _mappings.ElementAt(0), _mappings.ElementAt(2)
            });

            var mappingsXml = GetMappingsXml(_mappings);
            TopologyMappingsService.UpdateMappings(mappingsXml);

			Received.InOrder(() =>
			{
				TopologyManagerClient.UpdateObject(ArgIs(_mappings.ElementAt(0)));
				TopologyManagerClient.UpdateObject(ArgIs(_mappings.ElementAt(2)));
				TopologyManagerClient.SaveChanges(SaveChangesOptions.BatchWithSingleChangeset);
			});
			
			TopologyManagerClient.DidNotReceive().UpdateObject(ArgIs(_mappings.ElementAt(1)));
        }

        [Test]
        public void Should_not_call_SaveChanges_if_no_data()
        {
            TopologyMappingsService.UpdateMappings(null);

            TopologyManagerClient.DidNotReceive().SaveChanges();
        }

        [Test]
        public void Should_not_call_SaveChanges_if_no_mappings()
        {
            var mappingsXml = GetMappingsXml(new MappingData[0]);
            TopologyMappingsService.UpdateMappings(mappingsXml);

            TopologyManagerClient.DidNotReceive().SaveChanges();
        }

        [Test]
        public void Should_handle_Topology_Manager_exception()
        {
            const string errorMessage = "Can not save";

            var mappingsXml = GetMappingsXml(_mappings);

			TopologyManagerClient.SaveChanges(SaveChangesOptions.BatchWithSingleChangeset).Throws(new InvalidOperationException(errorMessage));

            var exception = Assert.Throws<TopologyManagerClient.DataServiceException>(() => TopologyMappingsService.UpdateMappings(mappingsXml));
            Assert.AreEqual(errorMessage, exception.Message);
        }

        private static MappingData ArgIs(MappingData mapping)
        {
            return Arg.Is<MappingData>(x =>
                x.Id == mapping.Id
                && x.WebApplicationId == mapping.WebApplicationId
                && x.RelativeUrl == mapping.RelativeUrl);
        }
    }
}
